@@ -41,6 +41,16 @@
 #include <sane/sane.h>
 #include <sane/saneopts.h>
 
+#define DBG_fatal 0
+#define DBG_error 1
+#define DBG_warning 2
+#define DBG_info 3
+#define DBG_debug 4
+
+#define DEBUG_NOT_STATIC
+#define BACKEND_NAME xscanimage
+#include "../include/sane/sanei_debug.h"
+
 int gsg_message_dialog_active = 0;
 
 /* forward declarations: */
@@ -65,7 +75,7 @@ unit_string (SANE_Unit unit)
       else if (d > 25.3 && d < 25.5)
 	return "in";
       return "mm";
-    case SANE_UNIT_MICROSECOND:	return "\265s";
+    case SANE_UNIT_MICROSECOND:	return "us";
     }
   return 0;
 }
@@ -425,6 +435,7 @@ scale_update (GtkAdjustment * adj_data, GSGDialogElement * elem)
   SANE_Word val, new_val;
   int opt_num;
   double d;
+  SANE_Status status;
 
   opt_num = elem - dialog->element;
   opt = sane_get_option_descriptor (dialog->dev, opt_num);
@@ -446,8 +457,14 @@ scale_update (GtkAdjustment * adj_data, GSGDialogElement * elem)
       return;
     }
   set_option (dialog, opt_num, &val, SANE_ACTION_SET_VALUE);
-  sane_control_option (dialog->dev, opt_num, SANE_ACTION_GET_VALUE, &new_val,
-		       0);
+  status = sane_control_option (dialog->dev, opt_num, SANE_ACTION_GET_VALUE, &new_val,
+				0);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_fatal, "scale_update: sane_control_option failed: %s\n", sane_strstatus (status));
+      return;
+    }
+
   if (new_val != val)
     {
       val = new_val;
@@ -877,8 +894,16 @@ panel_build (GSGDialog * dialog)
   int num_vector_opts = 0, *vector_opts;
 
   main_hbox = gtk_hbox_new (FALSE, 2);
+ 
+  if (dialog->twocolumn)
+     { 
+       option_vbox = gtk_hbox_new (FALSE, 2);  /* two column display */
+     }
+   else
+     {
+       option_vbox = gtk_vbox_new (FALSE, 2);  /* one column display */
+     }
 
-  option_vbox = gtk_vbox_new (FALSE, 2);
   gtk_box_pack_start (GTK_BOX (main_hbox), option_vbox, FALSE, FALSE, 0);
   gtk_widget_show (option_vbox);
 
@@ -1241,6 +1266,7 @@ gsg_create_dialog (GtkWidget * window, const char *device_name,
   dialog->param_change_callback = param_change_callback;
   dialog->param_change_arg = param_change_arg;
   dialog->advanced = preferences.advanced;
+  dialog->twocolumn = preferences.twocolumn_enabled;
 
   dialog->element = malloc (num_elements * sizeof (dialog->element[0]));
   memset (dialog->element, 0, num_elements * sizeof (dialog->element[0]));
@@ -1384,6 +1410,13 @@ gsg_set_tooltips (GSGDialog *dialog, int enable)
     gtk_tooltips_disable (dialog->tooltips);
 }
 
+void
+gsg_set_twocolumn (GSGDialog *dialog, int twocolumn)
+{
+  dialog->twocolumn = twocolumn;
+       panel_rebuild (dialog);
+}
+
 void
 gsg_set_sensitivity (GSGDialog *dialog, int sensitive)
 {
