@@ -79,6 +79,16 @@
 #include "preview.h"
 #include "preferences.h"
 
+#define DBG_fatal 0
+#define DBG_error 1
+#define DBG_warning 2
+#define DBG_info 3
+#define DBG_debug 4
+
+#define DEBUG_DECLARE_ONLY
+#define BACKEND_NAME xscanimage
+#include "../include/sane/sanei_debug.h"
+
 #ifndef PATH_MAX
 # define PATH_MAX	1024
 #endif
@@ -497,12 +507,19 @@ static void
 set_option_bool (Preview *p, int option, SANE_Bool value)
 {
   SANE_Handle dev;
+  SANE_Status status;
 
   if (option <= 0)
     return;
 
   dev = p->dialog->dev;
-  sane_control_option (dev, option, SANE_ACTION_SET_VALUE, &value, 0);
+  status = sane_control_option (dev, option, SANE_ACTION_SET_VALUE, &value, 0);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_fatal, "set_option_bool: sane_control_option failed: %s\n",
+	   sane_strstatus (status));
+      return;
+    }
 }
 
 static int
@@ -825,6 +842,7 @@ scan_done (Preview *p)
   gtk_widget_set_sensitive (p->cancel, FALSE);
   gtk_widget_set_sensitive (p->preview, TRUE);
   gsg_set_sensitivity (p->dialog, TRUE);
+  gtk_widget_set_sensitive (p->dialog->window->parent->parent->parent, TRUE);
 }
 
 static void
@@ -838,6 +856,7 @@ scan_start (Preview *p)
   gtk_widget_set_sensitive (p->cancel, TRUE);
   gtk_widget_set_sensitive (p->preview, FALSE);
   gsg_set_sensitivity (p->dialog, FALSE);
+  gtk_widget_set_sensitive (p->dialog->window->parent->parent->parent, FALSE);
 
   /* clear old preview: */
   memset (p->preview_row, 0xff, 3*p->preview_width);
@@ -1403,6 +1422,7 @@ preview_scan (Preview *p)
   int i;
   SANE_Status status;
 
+
   save_option (p, p->dialog->well_known.dpi,
 	       &p->saved_dpi, &p->saved_dpi_valid);
   for (i = 0; i < 4; ++i)
