@@ -492,6 +492,13 @@ pref_toggle_tooltips (GtkWidget *widget, gpointer data)
   gsg_set_tooltips (dialog, preferences.tooltips_enabled);
 }
 
+static void
+pref_toggle_twocolumn (GtkWidget *widget, gpointer data)
+{
+  preferences.twocolumn_enabled = (GTK_CHECK_MENU_ITEM (widget)->active != 0);
+  gsg_set_twocolumn (dialog, preferences.twocolumn_enabled);
+}
+
 static GtkWidget *
 build_preferences_menu (GSGDialog *dialog)
 {
@@ -517,6 +524,16 @@ build_preferences_menu (GSGDialog *dialog)
   gtk_widget_show (item);
   gtk_signal_connect (GTK_OBJECT (item), "toggled",
 		      (GtkSignalFunc) pref_toggle_tooltips, 0);
+  
+  /* twocolumn submenu: */
+
+  item = gtk_check_menu_item_new_with_label ("Show two column display");
+  gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (item),
+				 preferences.twocolumn_enabled);
+  gtk_menu_append (GTK_MENU (menu), item);
+  gtk_widget_show (item);
+  gtk_signal_connect (GTK_OBJECT (item), "toggled",
+		      (GtkSignalFunc) pref_toggle_twocolumn, 0);
 
   item = gtk_menu_item_new ();
   gtk_container_add (GTK_CONTAINER (menu), item);
