@@ -45,6 +45,7 @@
 #include <sane/saneopts.h>
 #include "../include/sane/sanei.h"
 
+#define DEBUG_DECLARE_ONLY
 #define BACKEND_NAME xscanimage
 #include "../include/sane/sanei_debug.h"
 
@@ -64,13 +65,13 @@
 
 # ifdef HAVE_LIBGIMP_GIMPFEATURES_H
 #  include <libgimp/gimpfeatures.h>
-# elif !defined(ENABLE_GIMP_1_3)
+# elif defined(ENABLE_GIMP_1_2)
 #  define GIMP_CHECK_VERSION(major, minor, micro) 0
 # endif /* HAVE_LIBGIMP_GIMPFEATURES_H */
 
-#ifdef ENABLE_GIMP_1_3
+#ifndef ENABLE_GIMP_1_2
 # define GIMP_HAVE_RESOLUTION_INFO
-#endif /* ENABLE_GIMP_1_3 */
+#endif /* !ENABLE_GIMP_1_2 */
 
 # ifdef GIMP_CHECK_VERSION
 #  if GIMP_CHECK_VERSION(1,1,25)
@@ -80,15 +81,20 @@
 #   include "xscanimage-gimp-1_0-compat.h" 
 #  endif
 # else
-#  ifndef ENABLE_GIMP_1_3
+#  ifdef ENABLE_GIMP_1_2
 /* we have the old gimp interface and need the compatibility header file */
 #   include "xscanimage-gimp-1_0-compat.h" 
-#  endif /* !ENABLE_GIMP_1_3 */
+#  endif /* ENABLE_GIMP_1_2 */
 # endif
 
 static void query (void);
+#ifndef ENABLE_GIMP_1_2
+static void run (const gchar *name, gint nparams, const GimpParam * param,
+		 gint *nreturn_vals, GimpParam ** return_vals);
+#else
 static void run (char *name, int nparams, GimpParam * param,
 		 int *nreturn_vals, GimpParam ** return_vals);
+#endif /* !ENABLE_GIMP_1_2 */
 
 GimpPlugInInfo PLUG_IN_INFO =
 {
@@ -114,6 +120,7 @@ enum
 static struct
   {
     GtkWidget *shell;
+    GtkWidget *menubar;
     GtkWidget *hruler;
     GtkWidget *vruler;
     GtkWidget *info_label;
@@ -194,6 +201,7 @@ encode_devname (const char *devname, int n, char *buf)
   const char *src;
   char ch;
 
+  DBG(DBG_debug, "encode_devname\n");
   limit = buf + n;
   for (src = devname, dst = buf; *src; ++src)
     {
@@ -228,6 +236,8 @@ encode_devname (const char *devname, int n, char *buf)
   if (dst >= limit)
     return -1;
   *dst = '\0';
+  DBG(DBG_debug, "encode_devname: %s --> %s\n", devname, buf);
+
   return 0;
 }
 
@@ -238,6 +248,7 @@ decode_devname (const char *encoded_devname, int n, char *buf)
   const char *src;
   char ch, val;
 
+  DBG(DBG_debug, "decode_devname\n");
   limit = buf + n;
   for (src = encoded_devname, dst = buf; *src; ++dst)
     {
@@ -279,6 +290,7 @@ decode_devname (const char *encoded_devname, int n, char *buf)
   if (dst >= limit)
     return -1;
   *dst = '\0';
+  DBG(DBG_debug, "deccode_devname: %s -> %s\n", encoded_devname, buf);
   return 0;
 }
 
@@ -296,7 +308,9 @@ query (void)
   char name[1024];
   size_t len;
   int i, j;
+  SANE_Status status;
 
+  DBG(DBG_debug, "query\n");
   gimp_install_procedure (
       "xscanimage",
       "Front-end to the SANE interface",
@@ -311,8 +325,19 @@ query (void)
       nargs, nreturn_vals,
       args, return_vals);
 
-  sane_init (0, 0);
-  sane_get_devices (&devlist, SANE_FALSE);
+  status = sane_init (0, 0);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_fatal, "query: sane_init failed: %s\n", sane_strstatus (status));
+      exit (1);
+    }
+
+  status = sane_get_devices (&devlist, SANE_FALSE);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_fatal, "query: sane_get_devices failed: %s\n", sane_strstatus (status));
+      exit (1);
+    }
 
   for (i = 0; devlist[i]; ++i)
     {
@@ -342,22 +367,30 @@ query (void)
 	 nargs, nreturn_vals, args, return_vals);
     }
   sane_exit ();
+  DBG(DBG_debug, "query: finished\n");
 }
 
+#ifndef ENABLE_GIMP_1_2
+static void
+run (const gchar *name, gint nparams, const GimpParam * param,
+     gint *nreturn_vals, GimpParam ** return_vals)
+#else
 static void
 run (char *name, int nparams, GimpParam * param,
      int *nreturn_vals, GimpParam ** return_vals)
+#endif /* !ENABLE_GIMP_1_2 */
 {
   static GimpParam values[2];
-#ifdef ENABLE_GIMP_1_3
+#ifndef ENABLE_GIMP_1_2
   GimpRunMode run_mode;
 #else
   GimpRunModeType run_mode;
-#endif /* ENABLE_GIMP_1_3 */
+#endif /* !ENABLE_GIMP_1_2 */
   char devname[1024];
   char *args[2];
   int nargs;
 
+  DBG(DBG_debug, "run\n");
   run_mode = param[0].data.d_int32;
   scan_win.mode = SANE_GIMP_EXTENSION;
 
@@ -365,6 +398,7 @@ run (char *name, int nparams, GimpParam * param,
   *return_vals = values;
 
   values[0].type = GIMP_PDB_STATUS;
+  values[0].data.d_status = GIMP_PDB_CALLING_ERROR;
 
   nargs = 0;
   args[nargs++] = "xscanimage";
@@ -380,6 +414,9 @@ run (char *name, int nparams, GimpParam * param,
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
+#ifndef ENABLE_GIMP_1_2
+      gimp_extension_ack();
+#endif /* !ENABLE_GIMP_1_2 */
       interface (nargs, args);
       values[0].data.d_status = GIMP_PDB_SUCCESS;
       break;
@@ -395,6 +432,7 @@ run (char *name, int nparams, GimpParam * param,
     default:
       break;
     }
+  DBG(DBG_debug, "run: finished\n");
 }
 
 static void
@@ -412,6 +450,7 @@ SANE_Word get_resolution (SANE_Handle dev)
   SANE_Int num_options, i;
   const SANE_Option_Descriptor *option_desc;
 
+  DBG(DBG_debug, "get_resolution\n");
   status = sane_control_option (dev, 0, SANE_ACTION_GET_VALUE, &num_options, 0);
   if (status != SANE_STATUS_GOOD)
     return 0;
@@ -425,8 +464,8 @@ SANE_Word get_resolution (SANE_Handle dev)
 	    if (strncmp (option_desc->name, SANE_NAME_SCAN_RESOLUTION, 
 			 sizeof(SANE_NAME_SCAN_RESOLUTION)) == 0)
 	      {
-		sane_control_option (dev, i, SANE_ACTION_GET_VALUE,
-				     &resolution, 0);
+		status = sane_control_option (dev, i, SANE_ACTION_GET_VALUE,
+					      &resolution, 0);
 		if (status == SANE_STATUS_GOOD) 
 		  {
 		    if (option_desc->type == SANE_TYPE_INT)
@@ -438,6 +477,7 @@ SANE_Word get_resolution (SANE_Handle dev)
 	      }
 	  }
     }      
+  DBG(DBG_debug, "get_resolution: finished\n");
   return 0; 
 }
 
@@ -455,13 +495,14 @@ update_param (GSGDialog *dialog, void *arg)
   SANE_Parameters params;
   gchar buf[200];
 
+  DBG(DBG_debug, "update_param\n");
   if (!scan_win.info_label)
     return;
 
   if (sane_get_parameters (gsg_dialog_get_device (dialog), &params)
       == SANE_STATUS_GOOD)
     {
-      double size = params.bytes_per_line * params.lines;
+      double size = (double) params.bytes_per_line * (double) params.lines;
       const char *unit = "B";
 
       if (params.lines == -1)
@@ -494,6 +535,7 @@ update_param (GSGDialog *dialog, void *arg)
 
   if (scan_win.preview)
     preview_update (scan_win.preview);
+  DBG(DBG_debug, "update_param: finished\n");
 }
 
 static void
@@ -502,6 +544,7 @@ pref_xscanimage_save (void)
   char filename[PATH_MAX];
   int fd;
 
+  DBG(DBG_debug, "pref_xscanimage_save\n");
   /* first save xscanimage-specific preferences: */
   gsg_make_path (sizeof (filename), filename, "xscanimage", "xscanimage",
 		 0, ".rc");
@@ -517,6 +560,7 @@ pref_xscanimage_save (void)
     }
   preferences_save (fd);
   close (fd);
+  DBG(DBG_debug, "pref_xscanimage_save: finished\n");
 }
 
 static void
@@ -525,6 +569,7 @@ pref_xscanimage_restore (void)
   char filename[PATH_MAX];
   int fd;
 
+  DBG(DBG_debug, "pref_xscanimage_restore\n");
   gsg_make_path (sizeof (filename), filename, "xscanimage", "xscanimage",
 		 0, ".rc");
   fd = open (filename, O_RDONLY);
@@ -535,11 +580,13 @@ pref_xscanimage_restore (void)
     }
   if (!preferences.filename)
     preferences.filename = strdup (OUTFILENAME);
+  DBG(DBG_debug, "pref_xscanimage_restore: finished\n");
 }
 
 static void
 quit_xscanimage (void)
 {
+  DBG (DBG_debug, "quit_xscanimage\n");
   if (scan_win.preview)
     {
       Preview *preview = scan_win.preview;
@@ -562,7 +609,7 @@ quit_xscanimage (void)
   if (scan_win.mode == SANE_GIMP_EXTENSION)
     gimp_quit ();
 #endif
-  DBG(DBG_debug, "xscanimage exiting.\n");
+  DBG (DBG_debug, "quit_xscanimage: exiting\n");
   exit (0);
 }
 
@@ -586,6 +633,8 @@ preview_window_destroyed (GtkWidget *widget, gpointer call_data)
 static void
 scan_preview (GtkWidget * widget, gpointer call_data)
 {
+  DBG(DBG_debug, "scan_preview\n");
+
   if (GTK_TOGGLE_BUTTON (widget)->active)
     {
       if (!scan_win.preview)
@@ -607,12 +656,15 @@ scan_preview (GtkWidget * widget, gpointer call_data)
       preview_destroy (scan_win.preview);
       scan_win.preview = 0;
     }
+  DBG(DBG_debug, "scan_preview: finished\n");
 }
 
 #ifdef HAVE_LIBGIMP_GIMP_H
 static void
 advance (void)
 {
+  DBG(DBG_debug, "advance\n");
+
   if (++scan_win.x >= scan_win.param.pixels_per_line)
     {
       int tile_height = gimp_tile_height ();
@@ -646,6 +698,7 @@ advance (void)
 	    scan_win.tile_offset = 0;
 	}
     }
+  DBG(DBG_debug, "advance: finished\n");
 }
 
 #endif /* HAVE_LIBGIMP_GIMP_H */
@@ -657,6 +710,7 @@ write_swapped_words(FILE *f, char *buf, guint len)
   char tmp;
   unsigned int i;
 
+  DBG(DBG_debug, "write_swapped_words\n");
   if (!len)
     return;
   if (scan_win.have_odd_byte)
@@ -682,6 +736,7 @@ write_swapped_words(FILE *f, char *buf, guint len)
 	  scan_win.odd_byte = buf[len - 1];
 	}
     }
+  DBG(DBG_debug, "write_swapped_words: finished\n");
 }
 
 static void
@@ -693,6 +748,7 @@ input_available (gpointer data, gint source, GdkInputCondition cond)
   SANE_Int len;
   int i;
 
+  DBG(DBG_debug, "input_available\n");
   while (1)
     {
       status = sane_read (dev, (unsigned char *) buf, sizeof (buf), &len);
@@ -896,6 +952,7 @@ input_available (gpointer data, gint source, GdkInputCondition cond)
 	  return;
 	}
     }
+  DBG(DBG_debug, "input_available: finished\n");
   return;
 
 #ifdef HAVE_LIBGIMP_GIMP_H
@@ -911,10 +968,12 @@ bad_depth:
 static void
 scan_done (void)
 {
+  DBG(DBG_debug, "scan_done\n");
   gsg_set_sensitivity (dialog, TRUE);
   if (scan_win.preview)
     gtk_widget_set_sensitive (scan_win.preview->preview, TRUE);
   gtk_widget_set_sensitive (scan_win.scan_button, TRUE);
+  gtk_widget_set_sensitive (scan_win.menubar, TRUE);
 
   if (scan_win.input_tag >= 0)
     {
@@ -957,12 +1016,15 @@ scan_done (void)
       scan_win.tile = 0;
     }
 #endif /* HAVE_LIBGIMP_GIMP_H */
+  DBG(DBG_debug, "scan_done: finished\n");
 }
 
 static void
 progress_cancel (void)
 {
+  DBG(DBG_debug, "progress_cancel\n");
   sane_cancel (gsg_dialog_get_device (dialog));
+  DBG(DBG_debug, "progress_cancel: done\n");
 }
 
 static void
@@ -979,10 +1041,13 @@ scan_start (void)
   char buf[256];
   int fd;
 
+  DBG(DBG_debug, "scan_start\n");
+
   gsg_set_sensitivity (dialog, FALSE);
   if (scan_win.preview)
     gtk_widget_set_sensitive (scan_win.preview->preview, FALSE);
   gtk_widget_set_sensitive (scan_win.scan_button, FALSE);
+  gtk_widget_set_sensitive (scan_win.menubar, FALSE);
 
 #ifdef HAVE_LIBGIMP_GIMP_H
   if (scan_win.mode == SANE_GIMP_EXTENSION && scan_win.tile)
@@ -1187,6 +1252,7 @@ scan_start (void)
 	gtk_main_iteration ();
       input_available (0, -1, GDK_INPUT_READ);
     }
+  DBG(DBG_debug, "scan_start: finished\n");
 }
 
 /* Invoked when the scan button is pressed */
@@ -1194,9 +1260,29 @@ static void
 scan_dialog (GtkWidget * widget, gpointer call_data)
 {
   char buf[256];
+  char testfilename[256];
   
+  DBG(DBG_debug, "scan_dialog\n");
   if (scan_win.mode == STANDALONE)
     {				/* We are running in standalone mode */
+      				/* test for pnm formats*/
+      strncpy (testfilename, preferences.filename, sizeof (testfilename));
+      testfilename [sizeof (testfilename)] = 0;
+      g_strreverse (testfilename);
+      if (! ((!strncmp(testfilename,"mnp.",4)) || 
+	     (!strncmp(testfilename,"mgp.",4)) ||
+	     (!strncmp(testfilename,"mbp.",4)) ||
+	     (!strncmp(testfilename,"mpp.",4)) ||
+	     (!strncmp(testfilename,"MNP.",4)) || 
+	     (!strncmp(testfilename,"MGP.",4)) ||
+	     (!strncmp(testfilename,"MBP.",4)) ||
+	     (!strncmp(testfilename,"MPP.",4)) ))
+	{
+	  snprintf (buf, sizeof (buf), "Failed to scan, wrong file extension, use pnm, pgm, pbm or ppm `%s'",
+		    preferences.filename);
+	  gsg_error (buf);
+	  return;
+	}
       scan_win.out = fopen (preferences.filename, "w");
       if (!scan_win.out)
 	{
@@ -1208,6 +1294,7 @@ scan_dialog (GtkWidget * widget, gpointer call_data)
     }
   gsg_sync (dialog);
   scan_start ();
+  DBG(DBG_debug, "scan_dialog: finished\n");
 }
 
 #if 0
@@ -1251,6 +1338,7 @@ files_build_menu (void)
 {
   GtkWidget *menu, *item;
 
+  DBG(DBG_debug, "files_build_menu\n");
   menu = gtk_menu_new ();
 
   item = gtk_menu_item_new ();
@@ -1263,6 +1351,7 @@ files_build_menu (void)
 		      (GtkSignalFunc) files_exit_callback, 0);
   gtk_widget_show (item);
 
+  DBG(DBG_debug, "files_build_menu: finished\n");
   return menu;
 }
 
@@ -1272,6 +1361,8 @@ pref_set_unit_callback (GtkWidget *widget, gpointer data)
   const char *unit = data;
   double unit_conversion_factor = 1.0;
 
+  DBG(DBG_debug, "pref_set_unit_callback\n");
+
   if (strcmp (unit, "cm") == 0)
     unit_conversion_factor = 10.0;
   else if (strcmp (unit, "in") == 0)
@@ -1284,6 +1375,7 @@ pref_set_unit_callback (GtkWidget *widget, gpointer data)
     preview_update (scan_win.preview);
 
   pref_xscanimage_save ();
+  DBG(DBG_debug, "pref_set_unit_callback: finished\n");
 }
 
 static void
@@ -1307,7 +1399,7 @@ update_double_callback (GtkWidget *widget, gpointer data)
     return;
 
   v = strtod (start, &end);
-  if (end > start && v > 0.0)
+  if (end > start)
     *valuep = v;
 }
 
@@ -1317,13 +1409,27 @@ preview_options_ok_callback (GtkWidget *widget, gpointer data)
   GtkWidget *dialog = data;
   char buf[1024];
 
+  DBG(DBG_debug, "preview_options_ok_callback\n");
+
+    				/* gamma min, max test*/
+  if (preferences.preview_gamma < 0.0 || preferences.preview_gamma > 255.0)
+    {  				
+      snprintf (buf, sizeof (buf), "Gamma value %g is < 0 or > 255, please change!",
+		preferences.preview_gamma);
+      gsg_warning (buf);
+      return;
+    }
+
+
   gtk_widget_destroy (dialog);
   pref_xscanimage_save ();
 
+
   snprintf (buf, sizeof (buf),
 	    "It is necessary to restart %s for the changes to take effect.",
 	    prog_name);
   gsg_warning (buf);
+  DBG(DBG_debug, "preview_options_ok_callback: finished\n");
 }
 
 static void
@@ -1340,6 +1446,7 @@ preview_options_dialog (GtkWidget *widget, gpointer data)
   GtkWidget *dialog, *vbox, *hbox, *button, *label, *text;
   char buf[64];
 
+  DBG(DBG_debug, "preview_options_dialog\n");
   dialog = gtk_dialog_new ();
   sprintf (buf, "%s preview options", prog_name);
   gtk_window_set_title (GTK_WINDOW (dialog), buf);
@@ -1413,6 +1520,7 @@ preview_options_dialog (GtkWidget *widget, gpointer data)
   gtk_widget_show (button);
 
   gtk_widget_show (dialog);
+  DBG(DBG_debug, "preview_options_dialog: finished\n");
 }
 
 static void
@@ -1421,6 +1529,7 @@ pref_device_save (GtkWidget *widget, gpointer data)
   char filename[PATH_MAX];
   int fd;
 
+  DBG(DBG_debug, "pref_device_save\n");
   gsg_make_path (sizeof (filename), filename,
 		 "xscanimage", 0, dialog->dev_name, ".rc");
   fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0666);
@@ -1436,6 +1545,7 @@ pref_device_save (GtkWidget *widget, gpointer data)
   gsg_sync (dialog);
   sanei_save_values (fd, dialog->dev);
   close (fd);
+  DBG(DBG_debug, "pref_device_save: finished\n");
 }
 
 static void
@@ -1444,6 +1554,7 @@ pref_device_restore (void)
   char filename[PATH_MAX];
   int fd;
 
+  DBG(DBG_debug, "pref_device_restore\n");
   gsg_make_path (sizeof (filename), filename,
 		 "xscanimage", 0, dialog->dev_name, ".rc");
   fd = open (filename, O_RDONLY);
@@ -1453,6 +1564,7 @@ pref_device_restore (void)
   close (fd);
 
   gsg_refresh_dialog (dialog);
+  DBG(DBG_debug, "pref_device_restore: finished\n");
 }
 
 static void
@@ -1471,6 +1583,14 @@ pref_toggle_tooltips (GtkWidget *widget, gpointer data)
   pref_xscanimage_save ();
 }
 
+static void
+pref_toggle_twocolumn (GtkWidget *widget, gpointer data)
+{
+  preferences.twocolumn_enabled = (GTK_CHECK_MENU_ITEM (widget)->active != 0);
+  gsg_set_twocolumn (dialog, preferences.twocolumn_enabled);
+  pref_xscanimage_save ();
+}
+
 static GtkWidget *
 pref_build_menu (void)
 {
@@ -1479,6 +1599,7 @@ pref_build_menu (void)
   GSList *units_group = NULL;
   double unit;
 
+  DBG(DBG_debug, "pref_build_menu\n");
   menu = gtk_menu_new ();
 
   /* advanced user option: */
@@ -1499,6 +1620,16 @@ pref_build_menu (void)
   gtk_widget_show (item);
   gtk_signal_connect (GTK_OBJECT (item), "toggled",
 		      (GtkSignalFunc) pref_toggle_tooltips, 0);
+  
+  /* two column submenu: */
+
+  item = gtk_check_menu_item_new_with_label ("Show two column display");
+  gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (item),
+				 preferences.twocolumn_enabled);
+  gtk_menu_append (GTK_MENU (menu), item);
+  gtk_widget_show (item);
+  gtk_signal_connect (GTK_OBJECT (item), "toggled",
+		      (GtkSignalFunc) pref_toggle_twocolumn, 0);
 
   /* length unit submenu: */
 
@@ -1566,6 +1697,7 @@ pref_build_menu (void)
 		      (GtkSignalFunc) pref_device_restore, 0);
   gtk_widget_show (item);
 
+  DBG(DBG_debug, "pref_build_menu: finished\n");
   return menu;
 }
 
@@ -1574,6 +1706,7 @@ browse_filename_callback (GtkWidget *widget, gpointer data)
 {
   char filename[1024];
 
+  DBG(DBG_debug, "browse_filename_callback\n");
   if (preferences.filename)
     {
       strncpy (filename, preferences.filename, sizeof (filename));
@@ -1587,6 +1720,7 @@ browse_filename_callback (GtkWidget *widget, gpointer data)
   if (preferences.filename)
     free ((void *) preferences.filename);
   preferences.filename = strdup (filename);
+  DBG(DBG_debug, "browse_filename_callback: finished\n");
 }
 
 static void
@@ -1602,9 +1736,10 @@ static void
 device_dialog (void)
 {
   GtkWidget *vbox, *hbox, *button, *frame, *scrolled_window, *dialog_window, *label, *text;
-  GtkWidget *menubar, *menubar_item;
+  GtkWidget *menubar_item;
   const gchar *devname;
 
+  DBG(DBG_debug, "device_dialog\n");
   /* first, restore xscanimage preferences */
   pref_xscanimage_restore ();
 
@@ -1623,24 +1758,24 @@ device_dialog (void)
 
   /* create the menubar */
 
-  menubar = gtk_menu_bar_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
+  scan_win.menubar = gtk_menu_bar_new ();
+  gtk_box_pack_start (GTK_BOX (vbox), scan_win.menubar, FALSE, FALSE, 0);
 
   /* "Files" submenu: */
   menubar_item = gtk_menu_item_new_with_label ("File");
-  gtk_container_add (GTK_CONTAINER (menubar), menubar_item);
+  gtk_container_add (GTK_CONTAINER (scan_win.menubar), menubar_item);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar_item),
 			     files_build_menu ());
   gtk_widget_show (menubar_item);
 
   /* "Preferences" submenu: */
   menubar_item = gtk_menu_item_new_with_label ("Preferences");
-  gtk_container_add (GTK_CONTAINER (menubar), menubar_item);
+  gtk_container_add (GTK_CONTAINER (scan_win.menubar), menubar_item);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar_item),
 			     pref_build_menu ());
   gtk_widget_show (menubar_item);
 
-  gtk_widget_show (menubar);
+  gtk_widget_show (scan_win.menubar);
 
   /* if we're running in standalone mode, provide a output filename box: */
   if (scan_win.mode == STANDALONE)
@@ -1756,6 +1891,7 @@ device_dialog (void)
 
   pref_device_restore ();	/* restore device-settings */
   gtk_widget_show (scan_win.shell);
+  DBG(DBG_debug, "device_dialog: finished\n");
 }
 
 static void
@@ -1765,13 +1901,15 @@ ok_choose_dialog_callback (void)
   device_dialog ();
 }
 
-static void
+static int
 select_device_callback (GtkWidget * widget, GdkEventButton *event,
 			gpointer data)
 {
   seldev = (long) data;
   if (event->type == GDK_2BUTTON_PRESS && event->button == 1)
     ok_choose_dialog_callback ();
+
+  return 0;
 }
 
 static gint32
@@ -1782,6 +1920,7 @@ choose_device (void)
   const SANE_Device *adev;
   gint i;
 
+  DBG(DBG_debug, "choose_device\n");
   choose_device_dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (choose_device_dialog), "Select device");
   gtk_signal_connect (GTK_OBJECT (choose_device_dialog), "delete_event",
@@ -1833,6 +1972,7 @@ choose_device (void)
   gtk_widget_show (button);
 
   gtk_widget_show (choose_device_dialog);
+  DBG(DBG_debug, "choose_device: finished\n");
   return 0;
 }
 
@@ -1853,18 +1993,20 @@ init (int argc, char **argv)
 {
   char filename[PATH_MAX];
   struct stat st;
+  SANE_Status status;
 
+  DBG(DBG_debug, "init\n");
   gtk_init (&argc, &argv);
 #ifdef HAVE_LIBGIMP_GIMP_H
   gtk_rc_parse (gimp_gtkrc ());
 
-# ifdef ENABLE_GIMP_1_3
-  /* GIMP 1.3 defines gimp_use_xshm() as a macro always returning TRUE
+# ifndef ENABLE_GIMP_1_2
+  /* GIMP 2.0 defines gimp_use_xshm() as a macro always returning TRUE
    * (in the compat header) */
   gdk_set_use_xshm (TRUE);
 # else
   gdk_set_use_xshm (gimp_use_xshm ());
-# endif /* ENABLE_GIMP_1_3 */
+# endif /* !ENABLE_GIMP_1_2 */
 #endif
 
   gsg_make_path (sizeof (filename), filename, 0, "sane-style", 0, ".rc");
@@ -1879,7 +2021,12 @@ init (int argc, char **argv)
 	gtk_rc_parse (filename);
     }
 
-  sane_init (0, 0);
+  status = sane_init (0, 0);
+  if (status != SANE_STATUS_GOOD)
+    {
+      DBG (DBG_fatal, "init: sane_init failed: %s\n", sane_strstatus (status));
+      exit (1);
+    }
 
   if (argc > 1)
     {
@@ -1923,7 +2070,13 @@ init (int argc, char **argv)
     {
       char * defdevname;
 
-      sane_get_devices (&devlist, SANE_FALSE);
+      status = sane_get_devices (&devlist, SANE_FALSE);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (DBG_fatal, "init: sane_get_devices failed: %s\n", sane_strstatus (status));
+	  exit (1);
+	}
+
       if ((defdevname = getenv("SANE_DEFAULT_DEVICE")) != NULL)
 	{
 	  int i;
@@ -1943,6 +2096,7 @@ init (int argc, char **argv)
 	    DBG (DBG_info, "default device is `%s'\n", defdevname);
 	}
     }
+  DBG(DBG_debug, "init: finished\n");
 }
 
 static void
@@ -1950,6 +2104,7 @@ interface (int argc, char **argv)
 {
   scan_win.info_label = NULL;
 
+  DBG(DBG_debug, "interface\n");
   init (argc, argv);
 
   for (ndevs = 0; devlist[ndevs]; ++ndevs);
@@ -1991,14 +2146,17 @@ interface (int argc, char **argv)
 	}
     }
   gtk_quit_flag = 1;
+  DBG(DBG_debug, "interface: now running gtk_main ()\n");
   gtk_main ();
   sane_exit ();
+  DBG(DBG_debug, "interface: finished\n");
 }
 
 int
 main (int argc, char **argv)
 {
   DBG_INIT();
+  DBG(DBG_debug, "main\n");
   DBG(DBG_error, "xscanimage (version: %s, package: %s) starting\n", VERSION,
       PACKAGE);
   little_endian = calc_little_endian();
@@ -2026,7 +2184,11 @@ main (int argc, char **argv)
 # ifdef HAVE_OS2_H
     set_gimp_PLUG_IN_INFO(&PLUG_IN_INFO);
 # endif
+# ifndef ENABLE_GIMP_1_2
+    result = gimp_main (&PLUG_IN_INFO, argc, argv);
+# else
     result = gimp_main (argc, argv);
+# endif /* !ENABLE_GIMP_1_2 */
     g_set_print_handler (old_print_func);
     g_set_printerr_handler (old_printerr_func);
     if (result)
@@ -2035,5 +2197,6 @@ main (int argc, char **argv)
 #else
   interface (argc, argv);
 #endif /* HAVE_LIBGIMP_GIMP_H */
+  DBG(DBG_debug, "main: finished\n");
   return 0;
 }
